/*
 * Convert from degrees (e.g. 360) to radians (e.g. 2*Pi)
 */
function toRadians(degrees) {
    return degrees * Math.PI / 180;
}

/*
 * Convert from radians (e.g. 2*Pi) to degrees (e.g. 360)
 */
function toDegrees(radians) {
    return radians * 180 / Math.PI;
}

/*
 * Convert from polar coordinates (e.g. r/radius and theta/degrees)
 * to cartesian coordinates (e.g. x and y).
 *
 * The angle provided should be in degrees, not radians!
 */
function toCartesian(r, degrees) {
    var theta = toRadians(degrees);
    var x = r * Math.cos(theta);
    var y = r * Math.sin(theta);

    return {"x": x, "y": y};
}

/*
 * Convert from cartesian coordinates (e.g. x and y) to polar
 * coordinates (e.g. r/radius and theta/degrees)
 *
 * The angle returned will be in degrees, not radians!
 */
function toPolar(x, y) {
    var r = Math.sqrt(x * x + y * y);
    var theta = Math.atan2(y, x);

    return {"r": r, "theta": toDegrees(theta)};
}

/*
 * A magical function that fixes the size and viewbox of an SVG
 * to fit an inner group with the specified padding. Both the
 * svg and group parameters need to be D3 selections.
 */
function fixBounds(svg, group, padding) {
    // svg and group should be d3 selections
    if (!(svg instanceof d3.selection) ||
        !(group instanceof d3.selection)) {
        console.log("Warning: Must have D3 selections to fix bounds.");
        return;
    }

    // get bounding and view box
    var bbox = group.node().getBBox();
    var view = [bbox.x, bbox.y, bbox.width, bbox.height];

    var x = -bbox.x + padding;
    var y = -bbox.y + padding;

    var w = bbox.width + padding * 2;
    var h = bbox.height + padding * 2;

    group.attr("viewBox", view.join(" "));
    group.attr("transform", "translate(" + x + ", " + y + ")");

    svg.attr("width", w);
    svg.attr("height", h);

    svg.attr("viewBox", [0, 0, w, h].join(" "));
    svg.attr("version", "1.1");
    svg.attr("xmlns", "http://www.w3.org/2000/svg");
    svg.attr("preserveAspectRatio", "xMinYMin meet");
}
